@@ -1,14 +1,21 @@
-if __name__ == "__main__":
-    import sys
+#    Copyright © 2021 Andrei Puchko
+#
+#    Licensed under the Apache License, Version 2.0 (the "License");
+#    you may not use this file except in compliance with the License.
+#    You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an "AS IS" BASIS,
+#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#    See the License for the specific language governing permissions and
+#    limitations under the License.
 
-    sys.path.insert(0, ".")
-    from q2rad.q2rad import main
-
-    main()
 
 from q2gui.q2model import Q2Model
 from q2gui.q2dialogs import q2AskYN
-from q2rad import Q2Form
+from q2rad.q2utils import Q2Form
 import json
 from q2rad.q2raddb import read_url, open_url
 
@@ -20,27 +27,24 @@ _ = gettext.gettext
 
 
 class Q2Market(Q2Form):
-    def __init__(self):
+    def __init__(self, title=""):
         super().__init__("q2Market")
         self.no_view_action = True
 
     def on_init(self):
         self.add_control("app_title", _("Name"), datatype="char", datalen=100)
         self.add_control("app_version", _("Version"), datatype="char", datalen=100)
-        self.add_control(
-            "app_description", _("Description"), datatype="char", datalen=100
-        )
+        self.add_control("app_description", _("Description"), control="text", datatype="char", datalen=100)
         self.add_control("app_url", _("Path"), datatype="char", datalen=100)
 
         q2market_catalogue_url = f"{self.q2_app.q2market_url}/q2market.json"
-        print(q2market_catalogue_url)
-        dd = read_url(q2market_catalogue_url).decode("utf-8")
-        data = json.loads(dd)
+        data = json.loads(read_url(q2market_catalogue_url).decode("utf-8"))
         rez = []
         for x in data:
-            rec = data[x]
-            rec["app_url"] = x
-            rez.append(rec)
+            if "app_title" in data[x]:
+                rec = data[x]
+                rec["app_url"] = x
+                rez.append(rec)
         model = Q2Model()
         model.set_records(rez)
         self.set_model(model)
