@@ -1,16 +1,24 @@
-if __name__ == "__main__":
-    import sys
+#    Copyright © 2021 Andrei Puchko
+#
+#    Licensed under the Apache License, Version 2.0 (the "License");
+#    you may not use this file except in compliance with the License.
+#    You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an "AS IS" BASIS,
+#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#    See the License for the specific language governing permissions and
+#    limitations under the License.
 
-    sys.path.insert(0, ".")
-    from q2rad.q2rad import main
-
-    main()
 
 from q2db.cursor import Q2Cursor
 from q2gui.q2model import Q2CursorModel
 from q2gui.q2dialogs import q2Mess, q2AskYN
 
-from q2rad import Q2Form
+from q2rad.q2utils import Q2Form
+from q2gui import q2app
 import gettext
 
 
@@ -18,13 +26,15 @@ _ = gettext.gettext
 
 
 class Q2Packages(Q2Form):
-    def __init__(self):
+    def __init__(self, title=""):
         super().__init__("Packages")
         self.no_view_action = True
 
     def on_init(self):
+        self.db = q2app.q2_app.db_logic
         self.add_control("package_name", _("Name"), datatype="char", datalen=100, pk="*")
-        self.add_control("package_version", _("Version"), datatype="char", datalen=10, disabled="*")
+        self.add_control("package_pipname", _("Pip name"), datatype="char", datalen=100)
+        self.add_control("package_version", _("Version"), datatype="char", datalen=10)
         self.add_control("comment", _("Comment"), datatype="text")
 
         cursor: Q2Cursor = self.q2_app.db_logic.table(table_name="packages")
@@ -33,9 +43,9 @@ class Q2Packages(Q2Form):
         self.set_model(model)
         self.add_action("/crud")
         # self.add_action("Imp", self.imp)
-        self.add_action("Install", self.install)
-        self.add_action("Uninstall", self.uninstall)
-        self.add_action("Versions", self.info)
+        self.add_action("Install", self.install, eof_disabled=True)
+        self.add_action("Uninstall", self.uninstall, eof_disabled=True)
+        self.add_action("Versions", self.info, eof_disabled=True)
 
     # def imp(self):
     #     __import__(self.r.package_name)
@@ -45,17 +55,21 @@ class Q2Packages(Q2Form):
 
     def uninstall(self):
         if q2AskYN(f"You are about tu uninstall package: {self.r.package_name}") == 2:
-            self.q2_app.pip_uninstall(self.r.package_name)
+            self.q2_app.pip_uninstall(
+                self.r.package_name if self.r.package_pipname == "" else self.r.package_pipname
+            )
 
     def install(self):
         version = (
             self.r.package_version
             if self.r.package_version
-            else self.q2_app.get_package_versions(self.r.package_name)[0]
+            else self.q2_app.get_package_versions(self.r.package_name, self.r.package_pipname)[0]
         )
         if version:
             try:
-                self.q2_app.pip_install(self.r.package_name, version)
+                self.q2_app.pip_install(
+                    self.r.package_name if self.r.package_pipname == "" else self.r.package_pipname, version
+                )
             except Exception:
                 q2Mess(_(f"pip install <b>{self.r.package_name}</b> error!"))
             finally:
@@ -64,7 +78,9 @@ class Q2Packages(Q2Form):
             q2Mess(f"Package <b>{self.r.package_name}</b> not found!")
 
     def info(self):
-        latest_version, current_version = self.q2_app.get_package_versions(self.r.package_name)
+        latest_version, current_version = self.q2_app.get_package_versions(
+            self.r.package_name, self.r.package_pipname
+        )
         # if not current_version:
         #     current_version = "Was not imported; "
         #     _cv = self.q2_app.code_runner(
