@@ -1,34 +1,42 @@
-if __name__ == "__main__":
-    import sys
-
-    sys.path.insert(0, ".")
-    from q2rad.q2rad import main
-
-    main()
+#    Copyright © 2021 Andrei Puchko
+#
+#    Licensed under the Apache License, Version 2.0 (the "License");
+#    you may not use this file except in compliance with the License.
+#    You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an "AS IS" BASIS,
+#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#    See the License for the specific language governing permissions and
+#    limitations under the License.
 
 
 from q2db.cursor import Q2Cursor
 from q2gui.q2model import Q2CursorModel, Q2Model
 from q2gui.q2widget import Q2Widget
 from q2gui.q2app import Q2Actions
-from q2rad.q2raddb import q2cursor
+from q2rad.q2utils import q2cursor, Q2_save_and_run, num
 from q2gui.q2utils import set_dict_default
 import json
 import gettext
 import re
-from q2rad import Q2Form
+from q2rad.q2utils import Q2Form
+from q2gui import q2app
 
 _ = gettext.gettext
-re_find_param = re.compile(r":\b\w+\b")
+re_find_param = re.compile(r"(?::\b\w+\b|\{.+\})")
 
 
-class Q2Queries(Q2Form):
-    def __init__(self):
+class Q2Queries(Q2Form, Q2_save_and_run):
+    def __init__(self, title=""):
         super().__init__("Queries")
         self.no_view_action = True
 
     def on_init(self):
         self.create_form()
+        self.db = q2app.q2_app.db_logic
 
         cursor: Q2Cursor = self.q2_app.db_logic.table(table_name="queries")
         model = Q2CursorModel(cursor)
@@ -48,13 +56,22 @@ class Q2Queries(Q2Form):
         self.add_control(
             "content",
             "",
-            datatype="bigtext",
+            datatype="longtext",
             control="codejson",
             nogrid=1,
             noform=1,
             # readonly=1,
         )
-        self.add_control("comment", _("Comment"), datatype="text", noform=1)
+        self.add_control("comment", _("Comment"), datatype="longtext", noform=1)
+        self.add_control("q2_time", "Time", datatype="int", noform=1, alignment=7)
+        self._add_save_and_run(save_only=True)
+        self._add_save_and_run_visible(save_only=True)
+
+    def before_form_build(self):
+        if self._save_and_run_control is None:
+            self._save_and_run_control = self.controls.get("save_and_run_actions_visible")
+            self.controls.delete("save_and_run_actions_visible")
+        self.system_controls.insert(2, self._save_and_run_control)
 
     def before_form_show(self):
         self.maximized = True
@@ -157,7 +174,10 @@ class Q2QueryEdit(Q2Form):
         self.query_list = Q2QueryList(self)
         self.param_list = Q2ParamList()
         self.actions = Q2Actions()
+        self._db = None
         self.actions.add_action("Run F4", self.query_list.sql_runner, hotkey="F4")
+        self.actions.add_action("Dataset|Show as JSON", self.query_list.show_dataset_json)
+        self.actions.add_action("Dataset|Save as JSON", self.query_list.save_dataset_json)
 
         self.actions.show_main_button = 0
         self.actions.show_actions = 0
@@ -165,12 +185,30 @@ class Q2QueryEdit(Q2Form):
         self.add_control("hot_key_action", actions=self.actions, control="toolbar")
         if self.add_control("/vs", tag="qev"):
             self.add_control("ql", "", widget=self.query_list, nogrid=1, migrate=0)
-            self.add_control(
-                "run_query_button",
-                "Run (F4)",
-                valid=self.query_list.sql_runner,
-                control="button",
-            )
+            if self.add_control("/h"):
+                self.add_control("/s")
+                self.add_control(
+                    "run_query_button",
+                    "Run (F4)",
+                    valid=self.query_list.sql_runner,
+                    control="button",
+                )
+                self.add_control("/s")
+                self.add_control(
+                    "show_json",
+                    "Show as JSON",
+                    valid=self.query_list.show_dataset_json,
+                    control="button",
+                )
+                self.add_control("/s")
+                self.add_control(
+                    "save_json",
+                    "Save as JSON",
+                    valid=self.query_list.save_dataset_json,
+                    control="button",
+                )
+                self.add_control("/s")
+            self.add_control("/")
             self.add_control("pl", "", widget=self.param_list, nogrid=1, migrate=0)
             self.add_control("/")
         self.add_control("code", control="codesql", nogrid=1, valid=self.sql_changed)
@@ -178,7 +216,10 @@ class Q2QueryEdit(Q2Form):
 
     def set_content(self, content):
         if isinstance(content, str):
-            content_json = json.loads(content)
+            if content:
+                content_json = json.loads(content)
+            else:
+                content_json = {}
         else:
             content_json = content
         self.query_list.set_content(content_json.get("queries", {"new_query": "select * from "}))
@@ -218,13 +259,25 @@ class Q2QueryList(Q2Form):
         self.query_editor_form = query_editor_form
         self.no_view_action = True
         self.add_control("name", "Name", datatype="char", datalen="50")
-        self.add_control("sql", "Sql", datatype="bigtext", nogrid=1, noform=1)
+        self.add_control("sql", "Sql", datatype="longtext", nogrid=1, noform=1)
         self.add_control("/")
         self.set_model(Q2Model())
         self.add_action("/crud")
         self.model.readonly = False
         self.last_current_row = -1
 
+    def after_form_show(self):
+        if self.crud_mode != "EDIT":
+            start_value = self.r.name
+            while True:
+                _pkvalue_list = re.split(r"([^\d]+)", start_value)
+                _base = "".join(_pkvalue_list[:-1])
+                _suffix = num(_pkvalue_list[-1]) + 1
+                start_value = f"{_base}{_suffix}"
+                if start_value not in [row["name"] for row in self.model.records]:
+                    break
+            self.s.name = start_value
+
     def set_content(self, content):
         self.model.reset()
         for x in content:
@@ -236,8 +289,48 @@ class Q2QueryList(Q2Form):
         params = re_find_param.findall(sql)
         for x in params:
             value = self.query_editor_form.param_list.get_param(x)
-            sql = sql.replace(x, f"'{value}'")
-        q2cursor(sql).browse()
+            if x[1] == "_":
+                sql = sql.replace(x, f"{value}")
+            else:
+                sql = sql.replace(x, f"'{value}'")
+        q2cursor(sql, q2_db=self.query_editor_form._db).browse()
+
+    def prepare_dataset_json(self):
+        dataset_json = {"params": {}}
+        for query in self.model.get_records():
+            sql = query["sql"]
+            params = re_find_param.findall(sql)
+            for x in params:
+                value = self.query_editor_form.param_list.get_param(x)
+                dataset_json["params"][x] = value
+                if x[1] == "_":
+                    sql = sql.replace(x, f"{value}")
+                else:
+                    sql = sql.replace(x, f"'{value}'")
+            cu = q2cursor(sql, q2_db=self.query_editor_form._db)
+            rez = []
+            for x in cu.records():
+                rez.append(x)
+            dataset_json[query["name"]] = rez
+        return dataset_json
+
+    def show_dataset_json(self):
+        dataset_json = self.prepare_dataset_json()
+
+        if dataset_json:
+            json_form = Q2Form()
+            json_form.add_control("json", "", control="codejson", data=json.dumps(dataset_json, indent=2))
+            json_form.ok_button = True
+            json_form.show_form("JSON datasets")
+
+    def save_dataset_json(self):
+        dataset_json = self.prepare_dataset_json()
+
+        if dataset_json:
+            json_file_name = self.q2_app.get_save_file_dialoq(filter="JSON(*.json)")[0]
+            if json_file_name:
+                json_file_name = self.validate_impexp_file_name(json_file_name, "json")
+                open(json_file_name, "w", encoding="utf8").write(json.dumps(dataset_json, indent=2))
 
     def sql_to_model(self, sql):
         self.model.update({"sql": sql}, self.current_row, refresh=False)
@@ -298,13 +391,13 @@ class Q2ParamList(Q2Form):
 
     def get_param(self, param):
         for x in self.model.records:
-            if x["name"] == param[1:]:
+            if x["name"] == (param[1:] if param.startswith(":") else param[1:-1]):
                 return x["value"]
         return ""
 
     def put_params(self, sql):
         params = set(re_find_param.findall(sql))
-        params = {x[1:] for x in params}
+        params = {x[1:] if x.startswith(":") else x[1:-1] for x in params}
         names = {}
         for x in range(len(self.model.records)):
             names[self.model.records[x]["name"]] = x
